# _*_coding:utf-8_*_

from django.apps import AppConfig


class FLowConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xj_flow'
    verbose_name = u'XX 流程系统'
    sort = 20
